var buffer = require('buffer');
var HTML5 = require('html5');

function Buffer() {
	this.data = '';
	this.start = this.committed = 0;
	var eof;
	this.__defineSetter__('eof', function(f) {
		eof = f
		if(eof) this.append("\0")
	})
	this.__defineGetter__('eof', function() { return eof })
	this.eof = false;
}

exports.Buffer = Buffer;

Buffer.prototype = {
	slice: function() {
		if(this.start >= this.data.length) {
			if(!this.eof) throw 'drain';
			return HTML5.EOF;
		}
		return this.data.slice(this.start, this.data.length);
	},
	char: function() {
		if(this.start >= this.data.length) {
			if(!this.eof) throw 'drain';
			return "\0";
		}
		HTML5.debug('buffer.char', this.start, this.data.length, this.eof, this.data[this.start])
		return this.data[this.start++];
	},
	advance: function(amount) {
		this.start += amount;
		if(this.start >= this.data.length) {	
			if(!this.eof) throw 'drain';
		} else {
			if(this.committed > this.data.length / 2) {
				// Sliiiide
				this.data = this.data.slice(this.committed);
				this.start = this.start - this.committed;
				this.committed = 0;
			}
		}
	},
	matchWhile: function(re) {
		if(this.eof && this.data.length == this.start) return '';
		var r = new RegExp("^"+re+"+");
		if(m = r.exec(this.slice())) {
			if(!this.eof && m[0].length == this.data.length - this.start) throw('drain');
			this.advance(m[0].length);
			return m[0];
		} else {
			return '';
		}
	},
	matchUntil: function(re) {
		HTML5.debug('buffer.matchUntil', 'start')
		if(m = new RegExp(re + (this.eof ? "|\u0" : "")).exec(this.slice())) {
			HTML5.debug('buffer.matchUntil', this.data.length, this.start, this.data.length, m.index)
			var t = this.data.slice(this.start, this.start + m.index);
			this.advance(m.index);
			return t.toString();
		} else {
			if(this.eof) return '';
			throw('drain');
		}
	},
	append: function(data) {
		this.data += data
	},
	shift: function(n) {
		if(!this.eof && this.start + n >= this.data.length) throw('drain');
		d = this.data.slice(this.start, this.start + n).toString();
		this.advance(n);
		return d;
	},
	peek: function(n) {
		if(!this.eof && this.start + n >= this.data.length) throw('drain');
		return this.data.slice(this.start, Math.min(this.start + n, this.data.length)).toString();
	},
	length: function() {
		return this.data.length - this.start;
	},
	unget: function(d) {
		this.start -= (d.length);
	},
	undo: function() {
		this.start = this.committed;
	},
	commit: function() {
		this.committed = this.start;
	}
}
